package effects;

import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;

/**
 * Effect to display image as Fisheye.
 * @author Johann Houszka
 *
 */
public class EffectFisheye extends Effect implements IEffect{

	private double height, width;
	
	/**
	 * Constructor of this class.
	 */
	public EffectFisheye() {
		super();
		this.setDisplayName("Fisheye");
	}
	
	/* (non-Javadoc)
	 * @see effects.Effect#applyEffect(int[], double, double)
	 */
	public int[] applyEffect(int[] srcpixels, double width, double height) {
		 // create the result data
	    int[] dstpixels = new int[(int)(width*height)];            
	    // for each row
	    for (int y=0;y<height;y++) {                                
	        // normalize y coordinate to -1 ... 1
	        double ny = ((2*y)/height)-1;                        
	        // pre calculate ny*ny
	        double ny2 = ny*ny;                                
	        // for each column
	        for (int x=0;x<width;x++) {                            
	            // normalize x coordinate to -1 ... 1
	            double nx = ((2*x)/width)-1;                    
	            // pre calculate nx*nx
	            double nx2 = nx*nx;
	            // calculate distance from center (0,0)
	            // this will include circle or ellipse shape portion
	            // of the image, depending on image dimensions
	            // you can experiment with images with different dimensions
	            double r = Math.sqrt(nx2+ny2);                
	            // discard pixels outside from circle!
	            if (0.0<=r&&r<=1.0) {                            
	                double nr = Math.sqrt(1.0-r*r);            
	                // new distance is between 0 ... 1
	                nr = (r + (1.0-nr)) / 2.0;
	                // discard radius greater than 1.0
	                if (nr<=1.0) {
	                    // calculate the angle for polar coordinates
	                    double theta = Math.atan2(ny,nx);         
	                    // calculate new x position with new distance in same angle
	                    double nxn = nr*Math.cos(theta);        
	                    // calculate new y position with new distance in same angle
	                    double nyn = nr*Math.sin(theta);        
	                    // map from -1 ... 1 to image coordinates
	                    int x2 = (int)(((nxn+1)*width)/2.0);        
	                    // map from -1 ... 1 to image coordinates
	                    int y2 = (int)(((nyn+1)*height)/2.0);        
	                    // find (x2,y2) position from source pixels
	                    int srcpos = (int)(y2*width+x2);            
	                    // make sure that position stays within arrays
	                    if (srcpos>=0 & srcpos < width*height) {
	                        // get new pixel (x2,y2) and put it to target array at (x,y)
	                        dstpixels[(int)(y*width+x)] = srcpixels[srcpos];    
	                    }
	                }
	            }
	        }
	    }
	    //return result pixels
	    return dstpixels;
	} 
	
	/* (non-Javadoc)
	 * @see effects.Effect#applyEffectToBufferedImage(java.awt.image.BufferedImage)
	 */
	@Override
	public BufferedImage applyEffectToBufferedImage(BufferedImage img) {
		final int width = img.getWidth();
		final int height = img.getHeight();
		final int[] srcpixels = new int[width*height];
		
		 // create the result data
	    int[] dstpixels = new int[(int)(width*height)];            
	    // for each row
	    for (int y=0;y<height;y++) {                                
	        // normalize y coordinate to -1 ... 1
	        double ny = ((2*y)/height)-1;                        
	        // pre calculate ny*ny
	        double ny2 = ny*ny;                                
	        // for each column
	        for (int x=0;x<width;x++) {                            
	            // normalize x coordinate to -1 ... 1
	            double nx = ((2*x)/width)-1;                    
	            // pre calculate nx*nx
	            double nx2 = nx*nx;
	            // calculate distance from center (0,0)
	            // this will include circle or ellipse shape portion
	            // of the image, depending on image dimensions
	            // you can experiment with images with different dimensions
	            double r = Math.sqrt(nx2+ny2);                
	            // discard pixels outside from circle!
	            if (0.0<=r&&r<=1.0) {                            
	                double nr = Math.sqrt(1.0-r*r);            
	                // new distance is between 0 ... 1
	                nr = (r + (1.0-nr)) / 2.0;
	                // discard radius greater than 1.0
	                if (nr<=1.0) {
	                    // calculate the angle for polar coordinates
	                    double theta = Math.atan2(ny,nx);         
	                    // calculate new x position with new distance in same angle
	                    double nxn = nr*Math.cos(theta);        
	                    // calculate new y position with new distance in same angle
	                    double nyn = nr*Math.sin(theta);        
	                    // map from -1 ... 1 to image coordinates
	                    int x2 = (int)(((nxn+1)*width)/2.0);        
	                    // map from -1 ... 1 to image coordinates
	                    int y2 = (int)(((nyn+1)*height)/2.0);        
	                    // find (x2,y2) position from source pixels
	                    int srcpos = (int)(y2*width+x2);            
	                    // make sure that position stays within arrays
	                    if (srcpos>=0 & srcpos < width*height) {
	                        // get new pixel (x2,y2) and put it to target array at (x,y)
	                        dstpixels[(int)(y*width+x)] = srcpixels[srcpos];    
	                    }
	                }
	            }
	        }
	    }
	    Graphics graphics = img.getGraphics();
	    WritableRaster raster = img.getRaster();
        raster.getDataElements(0, 0, width, height, srcpixels);
	    //return result pixels
        raster.setDataElements(0, 0, width, height, dstpixels);
        graphics.drawImage(img, 0, 0, null);
        
        return img;
	}
	
	/**
	 * Sets height of the image.
	 * @param double height
	 */
	public void setHeight(double height) {
		this.height = height;
	}
	
	/**
	 * Returns height of image.
	 * @return double height
	 */
	public double getHeight() {
		return height;
	}
	
	/**
	 * Sets width of image.
	 * @param double width
	 */
	public void setWidth(double width) {
		this.width = width;
	}
	
	/**
	 * Returns width of image.
	 * @return double width
	 */
	public double getWidth() {
		return width;
	}
}
