package model;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.InputStreamReader;
import java.util.ArrayList;

import player.Player;

import controller.PlayerUIListener;

/**
 * Class for loading and saving Subtitles.
 * @author Matthias Eigner
 *
 */
public class Subtitles {
	
	private ArrayList<Subtitle> subtitles;
	private String file;
	private Player player;
	
	/**
	 * Construcor for this class.
	 * @param file
	 * @param player
	 */
	public Subtitles(String file, Player player){
		this.file = file;
		if(!this.file.endsWith(".srt")) this.file += ".srt";
		this.player = player;
		this.parseSubtitles();
	}
	
	/**
	 * Save Subtitles to file.
	 * @return boolean 
	 */
	public boolean saveToFile() {
		String content = "";
		int i = 0;
		for(Subtitle stitle: this.subtitles){
			if(i==0) content += stitle.toFileFirst(++i);
			else content += stitle.toFile(++i);
		}
		
		try {
			File file_ = new File(this.file);
			 
			// if file doesnt exists, then create it
			if (!file_.exists()) {
				return false;
			} else {
				FileWriter fw = new FileWriter(file_.getAbsoluteFile());
				BufferedWriter bw = new BufferedWriter(fw);
				bw.write(content);
				bw.close();
				
				return true;
			}
		} catch(Exception e){
			return false;
		}
	}
	
	/**
	 * Parse subtitles from file to ArrayList.
	 */
	public void parseSubtitles() {
		try {
			FileInputStream fstream = new FileInputStream(this.file);
			DataInputStream in = new DataInputStream(fstream);
			BufferedReader br = new BufferedReader(new InputStreamReader(in));
			String strLine;
			this.subtitles = new ArrayList<Subtitle>();
			Subtitle tmpTitle = null;
			int firstLine = -1;
			
			String tmp0[], tmp1[], tmp2[];
			
			while((strLine = br.readLine()) != null) {
				//System.out.println(strLine);
				//leerzeile zwischen einzelnene subtitles
				if(strLine.equals("") || firstLine == -1) {
					if(tmpTitle != null) this.subtitles.add(tmpTitle);
					tmpTitle = new Subtitle();
					firstLine = 1;
				} 
				if(!strLine.equals("")){
					// set parameters
					if(strLine.contains("-->")) {
						tmp0 = strLine.split("-->");
						
						tmp1 = tmp0[0].trim().split(":");
						tmp2 = tmp1[2].trim().split(",");
						
						
						tmpTitle.setStart(PlayerUIListener.timeToMilli(Integer.valueOf(tmp1[0].trim()), 
																		Integer.valueOf(tmp1[1].trim()), 
																		Integer.valueOf(tmp2[0].trim()), 
																		Integer.valueOf(tmp2[1].trim())));
						
						tmp1 = tmp0[1].trim().split(":");
						tmp2 = tmp1[2].trim().split(",");
						

						tmpTitle.setEnd(PlayerUIListener.timeToMilli(Integer.valueOf(tmp1[0].trim()), 
																		Integer.valueOf(tmp1[1].trim()), 
																		Integer.valueOf(tmp2[0].trim()), 
																		Integer.valueOf(tmp2[1].trim())));
						
					}
					else {	
						//System.out.println(strLine+"-"+((firstLine == 1)?"true":"false"));
						if(firstLine == 1) firstLine = 0;
						else tmpTitle.addLine(strLine);
					}	
				}

			}
			if(tmpTitle != null) this.subtitles.add(tmpTitle);
			//System.out.println("Subtitle count: " + this.subtitles.size());
			in.close();
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Insert new subtitle into ArrayList.
	 * @param Subtitle s
	 */
	public void insertSubTitle(Subtitle s){
		Subtitle sub;
		int old_index=-1;
		for(int i=0; i < this.subtitles.size(); i++){
			sub = this.subtitles.get(i);
			if(sub.getStart() > s.getStart()){
				break;
			} else {
				old_index = i;
			}
		}
		this.subtitles.add(old_index+1, s);
		this.saveAndReloadSubtitles();
	}
	
	/**
	 * Edit existing subtitle.
	 * @param s
	 * @param start
	 * @param end
	 * @param value
	 * @return
	 */
	public boolean editSubTitle(Subtitle s, long start, long end, String value){
		if(this.subtitles.contains(s)){
			Subtitle s_ = this.subtitles.get(this.subtitles.indexOf(s));
			s_.setStart(start);
			s_.setEnd(end);
			s_.setValue(value);
			this.saveAndReloadSubtitles();
			return true;
		} else return false;
	}
	
	/**
	 * Delete subtitle from list.
	 * @param s
	 * @return
	 */
	public boolean deleteSubTitle(Subtitle s){
		if(this.subtitles.remove(s)) {
			this.saveAndReloadSubtitles();
			return true;
		} else return false;
	}
	
	/**
	 * Returns the ArrayList with subtitles.
	 * @return subtitles
	 */
	public ArrayList<Subtitle> getSubtitles() { return this.subtitles; }

	/**
	 * Save subtitles to file.
	 * Reload file and display changes.
	 * @return boolean
	 */
	public boolean saveAndReloadSubtitles() {
		//System.out.println(this.subtitles.size());
		this.saveToFile();
		//System.out.println("Subtitle: Saved to File");
		this.parseSubtitles();
		//System.out.println("Subtitle: Parsed");
		player.getMediaPlayer().setSubTitleFile(this.file);
		return true;
	}
	
	public String getFile() { return this.file; }
}
