package model;

import java.awt.image.BufferedImage;
import java.util.concurrent.CountDownLatch;

import player.Player;

import uk.co.caprica.vlcj.player.MediaPlayer;
import uk.co.caprica.vlcj.player.MediaPlayerEventAdapter;
import uk.co.caprica.vlcj.player.MediaPlayerFactory;

import com.sun.jna.NativeLibrary;

/**
 * From the libvlc-documentation - Class for creating thumbnails.
 * Generates a single thumbnail image from a media file.
 * @author Johann Houszka 0625523
 */
public class ThumbnailFactory {

	private static final String[] VLC_ARGS = { 
	"--intf", "dummy", /* no interface */
	"--vout", "dummy", /* we don't want video (output) */
	"--no-audio", /* we don't want audio (decoding) */
	"--no-video-title-show", /* nor the filename displayed */
	"--no-stats", /* no stats */
	"--no-sub-autodetect-file", /* we don't want subtitles */
	//"--no-inhibit", /* we don't want interfaces */
	"--no-disable-screensaver", /* we don't want interfaces */
	"--no-snapshot-preview", /* no blending in dummy vout */
	};

	private BufferedImage snapshot;
	private static final float VLC_THUMBNAIL_POSITION = 30.0f / 100.0f;
	
	/**
	 * Constructor of this class.
	 * @param sourceurl
	 * @param imageWidth
	 */
	public ThumbnailFactory(Player player, String sourceurl, int imageWidth) {
		NativeLibrary.addSearchPath("libvlc",
				player.getSettings().getLibPath("vlc"));
		System.setProperty("jna.library.path",
				player.getSettings().getLibPath("vlc"));
		
		MediaPlayerFactory factory = new MediaPlayerFactory(VLC_ARGS);
		MediaPlayer mediaPlayer = factory.newHeadlessMediaPlayer();

		final CountDownLatch inPositionLatch = new CountDownLatch(1);
		final CountDownLatch snapshotTakenLatch = new CountDownLatch(1);

		mediaPlayer.addMediaPlayerEventListener(new MediaPlayerEventAdapter() {
			@Override
			public void positionChanged(MediaPlayer mediaPlayer,
					float newPosition) {
				if (newPosition >= VLC_THUMBNAIL_POSITION * 0.9f) { 
					inPositionLatch.countDown();
				}
			}
			@Override
			public void snapshotTaken(MediaPlayer mediaPlayer, String filename) {
				System.out.println("snapshotTaken(filename=" + filename + ")");
				snapshotTakenLatch.countDown();
			}
		});

		if (mediaPlayer.startMedia(sourceurl)) {
			mediaPlayer.setPosition(VLC_THUMBNAIL_POSITION);
			try {
				inPositionLatch.await();
			} catch (InterruptedException e) {
				e.printStackTrace();
			} // Might wait forever if error

			this.snapshot = mediaPlayer.getSnapshot(imageWidth, 0);
			try {
				snapshotTakenLatch.await();
			} catch (InterruptedException e) {
				e.printStackTrace();
			} // Might wait forever if error

			mediaPlayer.stop();
		}
		mediaPlayer.release();
		factory.release();
	}
	
	/**
	 * Returns snapshot of video.
	 * @return BufferdImage snapshot
	 */
	public BufferedImage getSnapshot() {
		return this.snapshot;
	}
}
