package model;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.util.ArrayList;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import javax.imageio.ImageIO;
import javax.swing.JPanel;
import javax.swing.JSlider;

import view.PlayerMainPanel;

/**
 * Class to draw Timeline for UI.
 * @author Matthias Eigner
 *
 */
public class Timeline extends JPanel{
	
	/**
	 * DEFAULT ID
	 */
	private static final long serialVersionUID = 1L;
	
	private JSlider slider;
	public final static int maxResolution = 3000;
	
	private PlayerMainPanel mainPanel;
	
	private boolean loaded = false;
	
	//layout Images
	private BufferedImage img_playing_background;
	private BufferedImage img_playing_background_left;
	private BufferedImage img_playing_background_right;
	private BufferedImage img_playingbar_background;
	private BufferedImage img_playingbar_background_left;
	private BufferedImage img_playingbar_background_right;
	private BufferedImage img_playing_position;
	private BufferedImage sub_image;
	
	private ArrayList<stLine> subtitleLines;
	
    private final ScheduledExecutorService executorService = Executors.newSingleThreadScheduledExecutor();

    /**
     * Constructor of this class.
     * @param panel
     */
    public Timeline(PlayerMainPanel panel) {
		this.mainPanel = panel;
		slider = new JSlider();
		slider.setMinimum(0);
		slider.setMaximum(maxResolution);
		this.setBackground(new Color(220, 220, 220));
		
		loadImages();
		
        executorService.scheduleAtFixedRate(new HoverListener(this), 0L, 1L, TimeUnit.MICROSECONDS);
        this.addComponentListener(new ComponentListener() {
			public void componentResized(ComponentEvent arg0) {
				if(mainPanel.getUIListener().getSubtitles() != null) {
					generateSubTitlePositions(mainPanel.getUIListener().getSubtitles().getSubtitles());
				}
			}

			@Override
			public void componentHidden(ComponentEvent arg0) {
			}

			@Override
			public void componentMoved(ComponentEvent arg0) {
			}

			@Override
			public void componentShown(ComponentEvent arg0) {
			}
        });
		
	}
	
	/**
	 * Load images used to build the Timeline.
	 */
	private void loadImages() {
		try {
			img_playing_background = ImageIO.read(new File("img/layout/playing_background.png"));
			img_playing_background_left = ImageIO.read(new File("img/layout/playing_background_left.png"));
			img_playing_background_right = ImageIO.read(new File("img/layout/playing_background_right.png"));
			img_playingbar_background = ImageIO.read(new File("img/layout/playing_played_background.png"));
			img_playingbar_background_left = ImageIO.read(new File("img/layout/playing_played_background_left.png"));
			img_playingbar_background_right = ImageIO.read(new File("img/layout/playing_played_background_right.png"));
			img_playing_position = ImageIO.read(new File("img/layout/playing_active_pos.png"));
			
		} catch(Exception e) {e.printStackTrace(); }
	}
	
	/**
	 * Generate marks for subtitle positions.
	 * @param subtitles
	 */
	public void generateSubTitlePositions(ArrayList<Subtitle> subtitles){
		this.subtitleLines = new ArrayList<stLine>();
		float x, y;
		for(Subtitle stitle: subtitles){
			x = (float)((float)stitle.getStart() / (float)this.mainPanel.getMovieDuration());
			y = (float)((float)stitle.getEnd() / (float)this.mainPanel.getMovieDuration());
			this.subtitleLines.add(new stLine(x, y));
//			System.out.println("Subtitle: "+stitle.getStart()+"("+x+") - "+stitle.getEnd()+"("+y+") at length="+this.mainPanel.getMovieDuration());
		}
		this.repaintSubtitles();
	}
	
	/**
	 * Repaint the subtitle marks in the Timeline.
	 */
	public void repaintSubtitles() {
		this.sub_image = new BufferedImage(this.getWidth(), 4, BufferedImage.TYPE_INT_RGB);
		
		// draw subtitles
		if(this.subtitleLines != null){
			Graphics2D g = this.sub_image.createGraphics();
			g.setBackground(Color.white);
			for(stLine line: this.subtitleLines){
				g.setColor(Color.red);
				g.fill(new Rectangle2D.Double((double)(line.getStart()*this.getWidth()), 0, (double)(Math.round((line.getEnd()-line.getStart())*this.getWidth())), 4));
			//	g.draw3DRect(Math.round(line.getStart()*this.getWidth()), 0, Math.round((line.getEnd()-line.getStart())*this.getWidth()), 4, false);
			}
		}
		
		this.repaint();
	}
	
	/* (non-Javadoc)
	 * @see javax.swing.JComponent#paint(java.awt.Graphics)
	 */
	public void paint(Graphics g) {
		super.paint(g);
		
		g = paintPlaying(g);
		g = paintMousePosition(g);
		g = paintSubTitles(g);
	}
	
	/**
	 * Paints the subtitle marks.
	 * @param g
	 * @return
	 */
	public Graphics paintSubTitles(Graphics g) {
		int st_top = this.getHeight() - 20;
		g.drawImage(this.sub_image, 0, st_top, null);
		
		return g;
	}
	
	/**
	 * Paint the Timeline update during playing.
	 * @param g
	 * @return
	 */
	public Graphics paintPlaying(Graphics g) {
		int playing_top = this.getHeight() - 16;
		
		// draw black Border
		g.setColor(Color.black);
		g.drawRect(0, 0, this.getWidth(), this.getHeight());
		
		// draw background
		g.drawImage(img_playing_background, 0, playing_top, this.getWidth(), 16, null);
		g.drawImage(img_playing_background_left, 0, playing_top, 7, 16, null);
		g.drawImage(img_playing_background_right, this.getWidth()-7, playing_top, 7, 16, null);
		
		if(this.mainPanel.isLoaded()){
			int playposition = this.millis2Pixel(this.getValue(), maxResolution);
			
			g.drawImage(img_playingbar_background_left, 2, playing_top+3, 4, 10, null);
		
			if(playposition > 4) {
				g.drawImage(img_playingbar_background, 6, playing_top+3, Math.min(this.getWidth() - 12, playposition-6), 10, null);
				g.drawImage(img_playing_position, playposition, playing_top+2, 2, 12, null);
			} 
			
			if(playposition > this.getWidth() - 6){
				g.drawImage(img_playingbar_background_right, this.getWidth() - 6, playing_top+3, 4, 10, null);
			}
			
		}
		
		return g;
	}
	
	/**
	 * Paints the mouse Position in the Timeline.
	 * @param g
	 * @return
	 */
	public Graphics paintMousePosition(Graphics g){
		Point tmp = this.getMousePosition() != null ? this.getMousePosition() : null;
		if(tmp != null) {
			g.setColor(new Color(50, 50, 50));
			g.drawLine(tmp.x, 1, tmp.x, this.getHeight() - 16);
			g.drawImage(img_playing_position, tmp.x, this.getHeight() - 16+2, 2, 12, null);
		}
		
		return g;
	}
		
	/**
	 * Sets value.
	 * @param value
	 */
	public void setValue(int value) {
		slider.setValue(Math.round(
					(float) (
								(float) value / 
								(float) this.getWidth() * 
								(float) maxResolution
							)
						)
					);
		this.repaint();
	}
	
	/**
	 * Sets value as float.
	 * @param value
	 */
	public void setValueFloat(float value) {
		slider.setValue(Math.round(value));
		this.repaint();
	}

	/**
	 * Returns value.
	 * @return
	 */
	public int getValue() {
		return slider.getValue();
	}

	/**
	 * Flag to check if video is loaded.
	 * @return
	 */
	public boolean isLoaded() {
		return loaded;
	}

	/**
	 * Set the status of the boolean 'loaded'.
	 * @param loaded
	 */
	public void setLoaded(boolean loaded) {
		this.loaded = loaded;
	}
	
	/**
	 * ListenerClass for the Timeline.
	 * @author Matthias Eigner
	 *
	 */
	final class HoverListener implements Runnable {

		private Timeline timeline;
		public HoverListener(Timeline timeline){
			this.timeline = timeline;
		}
		
		@Override
		public void run() {
			// TODO Auto-generated method stub
			if(timeline.getMousePosition()!= null) {
				timeline.repaint();
			}
		}
		
	}
	
	/**
	 * Private class.
	 * @author Matthias Eigner
	 *
	 */
	private class stLine {
		private float start;
		private float end;
		
		public stLine(float start, float end){
			this.start = start;
			this.end = end;
		}

		public float getStart() {
			return start;
		}

		/*public void setStart(float start) {
			this.start = start;
		}*/

		public float getEnd() {
			return end;
		}

		/*public void setEnd(float end) {
			this.end = end;
		}*/

		
	}
	
	/**
	 * Converts milliseconds to pixel to display progress in Timeline.
	 * @param millis
	 * @param max
	 * @return
	 */
	private int millis2Pixel(int millis, long max){
		return Math.round(((float) ((float) this.getWidth() / (float) max)) * (float) millis);
	}
	
}
